#include "StaticMeshActor.h"

StaticMeshActor::StaticMeshActor(Mesh mesh, Mat4f modelMatrix, Texture2D baseColorTexture, Texture2D normalTexture, Texture2D metallicTexture, Texture2D roughnessTexture, Texture2D ambientOcclusionTexture, Vec4f diffuseColor, float shininess, std::string name)
    : mesh(mesh)
    , baseColorTexture(baseColorTexture)
    , normalTexture(normalTexture)
    , metallicTexture(metallicTexture)
    , roughnessTexture(roughnessTexture)
    , ambientOcclusionTexture(ambientOcclusionTexture)
    , modelMatrix(modelMatrix)
    , diffuseColor(diffuseColor)
    , shininess(shininess)
    , name(name)
{}

void StaticMeshActor::render(Mat4f view, Mat4f projection, Shader shader){
    // Bind the base color texture if available:
    if(!baseColorTexture.isDummy()){
        baseColorTexture.bind(0);
        shader.setUniform("baseColorTexture", 0);
    }
    shader.setUniform("hasBaseColorTexture", !baseColorTexture.isDummy());

    // Bind the metallic texture if available:
    if(!metallicTexture.isDummy()){
        metallicTexture.bind(1);
        shader.setUniform("metallicTexture", 1);
    }
    shader.setUniform("hasMetallicTexture", !metallicTexture.isDummy());

    // Bind the roughness texture if available:
    if(!roughnessTexture.isDummy()){
        roughnessTexture.bind(2);
        shader.setUniform("roughnessTexture", 2);
    }
    shader.setUniform("hasRoughnessTexture", !roughnessTexture.isDummy());

    // Bind the normal texture if available:
    if(!normalTexture.isDummy()){
        normalTexture.bind(3);
        shader.setUniform("normalTexture", 3);
    }
    shader.setUniform("hasNormalTexture", !normalTexture.isDummy());

    // Bind the normal texture if available:
    if(!ambientOcclusionTexture.isDummy()){
        ambientOcclusionTexture.bind(4);
        shader.setUniform("ambientOcclusionTexture", 4);
    }
    shader.setUniform("hasAmbientOcclusionTexture", !ambientOcclusionTexture.isDummy());

    // Color of the mesh:
    shader.setUniform("materialColor", diffuseColor, 3);

    // Shininess of the mesh:
    shader.setUniform("materialShininess", shininess);

    // Assume that our world mesh is already correctly transformed in the world
    // so we upload just the identity matrix:
    shader.setUniform("model", modelMatrix);

    // Upload our view matrix, which performs the transformation to view the
    // world from the 'perspective' of something, e.g. a camera or a pawn:
    shader.setUniform("view", view);

    // Upload the projection matrix (used in vertex shader):
    shader.setUniform("projection", projection);

    // Draw our triangle:
    mesh.render();
}

Mat4f StaticMeshActor::getModelMatrix(){
    return modelMatrix;
}
