// © 2022, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

// Include this file only once when compiling:
#pragma once

// Include the std::vector:
#include <vector>

// Include our Mat4f class:
#include "src/math/Mat4f.h"

// Include our Mesh class:
#include "src/primitives/Triangle.h"

// Include our Mesh class:
#include "src/Mesh.h"

// Include our Shader class:
#include "src/Shader.h"

// Include our Texture2D class:
#include "src/Texture2D.h"

// Include our world class:

/**
 * This is a spinning top class which represents a renderable
 * object in the world.
 */
class StaticMeshActor {
private:


    // A texture used for rendering:
    Texture2D baseColorTexture;

    // A normal texture used for rendering:
    Texture2D normalTexture;

    // A metallic texture used for rendering:
    Texture2D metallicTexture;

    // A roughness texture used for rendering:
    Texture2D roughnessTexture;

    // A ambient occlusion texture used for rendering:
    Texture2D ambientOcclusionTexture;

    // Model matrix which defines the position, rotation and
    // scale of the object in the world:
    Mat4f modelMatrix;

    // Diffuse color in which the mesh is rendered:
    Vec4f diffuseColor;

    // Shininess:
    float shininess;

public:
    // The mesh of this spinning top.
    Mesh mesh;
    /**
     * Creates a static mesh actor
     */
    StaticMeshActor(Mesh mesh, Mat4f modelMatrix, Texture2D baseColor = Texture2D(), Texture2D normal = Texture2D(), Texture2D metallic = Texture2D(), Texture2D roughness = Texture2D(), Texture2D ambientOcclusion = Texture2D(), Vec4f diffuseColor = Vec4f(1.f,1.f,1.f,1.f), float shininess = 128.f, std::string name ="");

    std::string name;
    /**
     * This function renders the object with the view and projection
     * matrix and the given shader.
     */
    void render(Mat4f view, Mat4f projection, Shader shader);

    /**
     * Returns the model matrix.
     */
    Mat4f getModelMatrix();
};
